#!/usr/bin/env python3

# check_mailstorelicence.py - Checks the amount of used licences.
#
# Copyright (c) 2012 - 2017 MailStore Software GmbH
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicence, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import sys
import argparse
import mailstore

if __name__ == "__main__":

    # Initialize command line argument parser
    parser = argparse.ArgumentParser(description="Checks the amount of used licenses.", 
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument("--username", "-u",
                        default="admin",
                        help="Mailstore SPE Username")

    parser.add_argument("--password", "-p",
                        default="admin",
                        help="Mailstore SPE Password")

    parser.add_argument("--host", "-H",
                        default="localhost",
                        help="Mailstore SPE Hostname/IP")

    parser.add_argument("--port", "-P",
                        type=int,
                        default=8474,
                        help="Mailstore SPE Management API Port")

    parser.add_argument("--instance", "-i",
                        default=None,
                        required=True,
                        help="Instance ID")

    parser.add_argument("--warning", "-w",
                        type=int,
                        default=None,
                        required=True,
                        help="Warning Threshold")

    parser.add_argument("--critical", "-c",
                        type=int,
                        default=None,
                        required=True,
                        help="Critcal Threshold")
    
    args = parser.parse_args()

    # Connect to MailStore SPE Management API
    api = mailstore.spe.Client(username=args.username, password=args.password, host=args.host, port=args.port,
                               ignoreInvalidSSLCerts=True)

    # Figure out how many licenses are currently in use and available
    try:
        instanceData = api.CreateLicenseRequest()["result"]["instanceData"]
        instanceID = [instance for instance in instanceData if instance["id"] == args.instance]
        usedLicenses = instanceID[0]["usedLicenseCount"]
    except Exception:
        print("UNKNOWN - Cannot retrieve licensing information")
        sys.exit(3)

    # Finally determine the status to return
    if usedLicenses >= args.critical:
        print("CRITICAL - Number of used licenses ({0}) is above or equal to critical threshold ({1})|"
              "UsedLicenses={0}".format(usedLicenses, args.critical))
        sys.exit(2)
    elif usedLicenses >= args.warning:
        print("WARNING - Number of used licenses ({0}) is above or equal to warning threshold ({1})|"
              "UsedLicenses={0}".format(usedLicenses, args.warning))
        sys.exit(1)
    else:
        print("OK - Number of used licenses: {0}|UsedLicenses={0}".format(usedLicenses))
        sys.exit(0)
