# mergeFolders.py - When many pst files have been archived, you will see one branch
#                   for each PST file in the archive.
#                   This scripts merges these branches-
#
# Copyright (c) 2012, 2013, 2014, 2015 MailStore Software GmbH
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# MailStore Management API connection settings
HOST = "localhost"
USER = "admin"
PASS = "admin"
PORT = 8474
IGNORESSLWARNINGS = True

# Enter the instance id here.
instanceid = "test"

# 'operation' is either "Exchange" [sic!], then every folder will be merged into the first exchange archive folder.
# Be aware that 'first' is the first one that is returned in a hashed dictionary, which is not necessarily the first one
# in alphabetical order.
# Use any other name (eg "Outlook") then all folders will be merged into that folder.
# operation = "Exchange"
operation = "Outlook"

# Enter a regular expression here. Only folders that match that expression will be renamed.
# Example:
# Folders that start with with the letters "Outlook" = "^Outlook", any folder = "."
# regex = "."
regex = "^Outlook"

# When file is an exisiting file, only folders of users of that file will be merged
# When file is "" all users folders will be merged.
# One user per line in 'file'.
file = ""
# file = "users.txt"

# ################## DON'T CHANGE ANYTHING BELOW THIS LINE ###################
import mailstore
import re


def move_folder(oldfolder, newfolder):
        print("renaming folder {} -> {}".format(oldfolder, newfolder))
        status = api.MoveFolder(instanceid, oldfolder, newfolder, False)
        # print logOutput messages
        while True:
            print(status["logOutput"] if status["logOutput"] else "", end="", flush=True)
            if status["statusCode"] != "running":
                break
            status = api.GetStatus(status)
        print(status["statusCode"])


def rename():
    # Get ChildFolders from MailStore.
    folders = api.GetChildFolders(instanceid, maxLevels=2)["result"]["childFolders"]
    # Test, if file is given.
    if file != "":
        with open(file) as f:
            file_users = [user.strip() for user in f.readlines()]
            # Filter, whether archive is also mentioned in 'file'.
            folders = [f for f in folders if f["name"] in file_users]
    # Walk over all archive folders.
    for folder in folders:        
        print("="*60)
        print("user : {}".format(folder["name"]))
        if folder["hasChildFolders"]:
            # Get ChildFolders that match regular expression.
            subfolders = [childFolder["name"] for childFolder in folder["childFolders"] if
                          foldermatch.match(childFolder["name"])]
            # Test, wether folders shall be moved in the first Exchange folder found.
            # This is not necessarily the first in alphabetical order.
            if operation == "Exchange":
                try:
                    # Get first Exchange folder.
                    target_folder = [childFolder["name"] for childFolder in folder["childFolders"] if
                                     "exchange" in childFolder["name"].lower()][0]
                except Exception as e:
                    print("no exchange folder found {}".format(e))
                    continue
                else:
                    print("exchange folder : {}".format(target_folder))
            # Else, use the given name as target folder.
            else:
                target_folder = operation
            # Remove target folder from source folder list, because there is no need to move that folder.
            # It might not be present, therefore try...
            try:
                subfolders.remove(target_folder)
            except ValueError:
                pass
            # Do the actual moving with the remaining source folders.
            for subfolder in subfolders:
                move_folder(folder["name"] + "/" + subfolder, folder["name"] + "/" + target_folder)
        else:
            print("no subfolders found")


api = mailstore.spe.Client(USER, PASS, HOST, PORT, ignoreInvalidSSLCerts=IGNORESSLWARNINGS)
foldermatch = re.compile(regex)

if __name__ == "__main__":
    rename()
    print("done")
