#!/usr/bin/env python3 

# rename_inbox.py - Renames all 'Inbox' folders to 'INBOX'.
#
# Copyright (c) 2012, 2013, 2014, 2015 MailStore Software GmbH
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# MailStore Management API connection settings
HOST = "localhost"
USER = "admin"
PASS = "admin"
PORT = 8474
IGNORESSLWARNINGS = True

INSTANCEID = "test"

# Only change something here, if you know what you do.

sourceInboxname = 'Inbox'
targetInboxname = 'INBOX'

# ################## DON'T CHANGE ANYTHING BELOW THIS LINE ###################
import mailstore


# Callback function for the API.
def printLogOutput(values):
    print(values, flush=True)


# Replace the most right occurrence of 'old'.
def rreplace(folder, old, new):
    return new.join(folder.rsplit(old, 1))


# This function renames the foldername that is passed from process_folders.
def rename_inbox(foldername):
    global api
    if foldername.endswith('/' + sourceInboxname):
        # targetfolder = foldername.replace(sourceInboxname, targetInboxname)
        targetfolder = rreplace(foldername, sourceInboxname, targetInboxname)
        print("+---------------------------------")
        print("|renaming folder {} -> {}".format(foldername, targetfolder))
        print(api.MoveFolder(INSTANCEID, foldername, targetfolder))
        print("+---------------------------------")
    else:
        print(foldername)


# This functions recurses into all subfolders.
def process_folders(folders, external_function=print, deepness=0, showpath=1, path=""):
    for folder in folders:
        folder_print_name = folder["name"] if not showpath else path + folder["name"]
        # output = " " * deepness * 4 + folder_print_name
        external_function(folder_print_name)
        if folder["hasChildFolders"]:
            try:
                folder["childFolders"]
            except KeyError:
                pass
            else:
                process_folders(folder["childFolders"], external_function, deepness + 1 if not showpath else deepness,
                                showpath, path=folder_print_name + "/")


api = mailstore.spe.Client(USER, PASS, HOST, PORT, callbackStatus=printLogOutput,
                           ignoreInvalidSSLCerts=IGNORESSLWARNINGS)
process_folders(api.GetChildFolders(INSTANCEID, maxLevels=3)["result"]["childFolders"], rename_inbox)
